/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2013 Torus Knot Software Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
-----------------------------------------------------------------------------
*/

#ifndef __SampleBrowser_Android_H__
#define __SampleBrowser_Android_H__

#ifdef OGRE_STATIC_LIB
#  ifdef OGRE_BUILD_RENDERSYSTEM_GL
#    define OGRE_STATIC_GL
#  endif
#  ifdef OGRE_BUILD_RENDERSYSTEM_GLES
#    define OGRE_STATIC_GLES
#    undef USE_RTSHADER_SYSTEM
#  endif
#  ifdef OGRE_BUILD_RENDERSYSTEM_GLES2
#undef OGRE_STATIC_GLES
#    define USE_RTSHADER_SYSTEM
#    define OGRE_STATIC_GLES2
#  endif
#  if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT
#    ifdef OGRE_BUILD_RENDERSYSTEM_D3D9
#		define OGRE_STATIC_Direct3D9
#    endif
// dx11 will only work on vista and above, so be careful about statically linking
#    ifdef OGRE_BUILD_RENDERSYSTEM_D3D11
#      define OGRE_STATIC_Direct3D11
#    endif
#  endif

#  ifdef OGRE_BUILD_PLUGIN_BSP
#  define OGRE_STATIC_BSPSceneManager
#  endif
#  ifdef OGRE_BUILD_PLUGIN_PFX
#  define OGRE_STATIC_ParticleFX
#  endif
#  ifdef OGRE_BUILD_PLUGIN_CG
#  define OGRE_STATIC_CgProgramManager
#  endif

#  ifdef OGRE_USE_PCZ
#    ifdef OGRE_BUILD_PLUGIN_PCZ
#    define OGRE_STATIC_PCZSceneManager
#    define OGRE_STATIC_OctreeZone
#    endif
#  else
#    ifdef OGRE_BUILD_PLUGIN_OCTREE
#    define OGRE_STATIC_OctreeSceneManager
#  endif
#     endif
#  include "OgreStaticPluginLoader.h"
#endif

#include "OIS.h"
#include <android_native_app_glue.h>
#include <android/log.h>
#include <EGL/egl.h>
#include "OgrePlatform.h"
#include "TinyOgre.h"
#include "Android/OgreAndroidEGLWindow.h"

#ifdef USE_RTSHADER_SYSTEM
#   include "OgreRTShaderSystem.h"
#endif

#ifdef OGRE_STATIC_LIB
#   include "OgreStaticPluginLoader.h"
#endif

#if OGRE_PLATFORM != OGRE_PLATFORM_ANDROID
#   error This header is for use with Android only
#endif

#define LOGI(...) ((void)__android_log_print(ANDROID_LOG_INFO, "Ogre", __VA_ARGS__))
#define LOGW(...) ((void)__android_log_print(ANDROID_LOG_WARN, "Ogre", __VA_ARGS__))

using namespace Ogre;
namespace OgreBites
{
	//class OgreAndroidBridge;

	/*=============================================================================
	| Ogre Android bridge
	=============================================================================*/
	class OgreAndroidBridge
	{
	public:
		static void init(struct android_app* state)
		{			
			__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "samplebrowser_android.h OgreAndroidBridge init called");
			//registering the app window state handler
			state->onAppCmd = &OgreAndroidBridge::handleCmd;

			if(mInit)
				return;

			mRoot = new Ogre::Root();
#ifdef OGRE_STATIC_LIB
			mStaticPluginLoader = new Ogre::StaticPluginLoader();
			mStaticPluginLoader->load();
#else
#error "need ogre static libs"
#endif
			__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "samplebrowser_android.h OgreAndroidBridge init StaticPluginLoader loaded");
			__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "samplebrowser_android.h OgreAndroidBridge init setting rendersystem");
			
			mRoot->setRenderSystem(mRoot->getAvailableRenderers().at(0));
			
			__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "samplebrowser_android.h OgreAndroidBridge init root init");
			mRoot->initialise(false);
			
			mInit = true;
			
		}

		static void shutdown()
		{
			if(!mInit)
				return;

			mInit = false;

			if(mBrowser)
			{
				mBrowser->closeApp();
				OGRE_DELETE mBrowser;
				mBrowser = NULL;
			}

			OGRE_DELETE mRoot;
			mRoot = NULL;
			mRenderWnd = NULL;

#ifdef OGRE_STATIC_LIB
			mStaticPluginLoader->unload();
			delete mStaticPluginLoader;
			mStaticPluginLoader = NULL;
#endif
		}
		static void handleCmd(struct android_app* app, int32_t cmd)
		{
			switch (cmd) 
			{
			case APP_CMD_SAVE_STATE:
				break;
			case APP_CMD_INIT_WINDOW:
				if (app->window && mRoot)
				{
					__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "OgreAndroidBridge handleCmd APP_CMD_INIT_WINDOW");
					AConfiguration* config = AConfiguration_new();
					AConfiguration_fromAssetManager(config, app->activity->assetManager);

					if (!mRenderWnd) 
					{
						Ogre::NameValuePairList opt;
						opt["externalWindowHandle"] = Ogre::StringConverter::toString((int)app->window);
						opt["androidConfig"] = Ogre::StringConverter::toString((int)config);

						mRenderWnd = Ogre::Root::getSingleton().createRenderWindow("OgreWindow", 0, 0, false, &opt);

						if(!mBrowser)
						{
							mBrowser = OGRE_NEW TinyOgre();
							mBrowser->initAppForAndroid(mRenderWnd, app, NULL, NULL);
							mBrowser->initApp();

							//mInputInjector = new AndroidInputInjector(mBrowser, mTouch, mKeyboard);
						}
					}
					else
					{
						static_cast<AndroidEGLWindow*>(mRenderWnd)->_createInternalResources(app->window, config);
					}

					AConfiguration_delete(config);
				}
				break;
			case APP_CMD_TERM_WINDOW:
				if(mRoot && mRenderWnd)
					static_cast<AndroidEGLWindow*>(mRenderWnd)->_destroyInternalResources();
				break;
			case APP_CMD_GAINED_FOCUS:
				break;
			case APP_CMD_LOST_FOCUS:
				break;
			case APP_CMD_CONFIG_CHANGED:
				break;
			}
		}
		static void go(struct android_app* state)
		{
			__android_log_write(ANDROID_LOG_INFO, "DEBUGGING", "samplebrowser_android.h OgreAndroidBridge go called - this starts a massive loop");
			int ident, events;
			struct android_poll_source* source;

			while (true)
			{
				while ((ident = ALooper_pollAll(0, NULL, &events, (void**)&source)) >= 0)
				{
					if (source != NULL)
						source->process(state, source);

					if (state->destroyRequested != 0)
						return;
				}

				if(mRenderWnd != NULL && mRenderWnd->isActive())
				{
					mRenderWnd->windowMovedOrResized();
					mRoot->renderOneFrame();
					__android_log_write(ANDROID_LOG_VERBOSE, "DEBUGGING", "OMGLOOP");
				}
			}
		}

		static Ogre::RenderWindow* getRenderWindow()
		{
			return mRenderWnd;
		}

	private:
		static TinyOgre* mBrowser;
		static Ogre::RenderWindow* mRenderWnd;
		static Ogre::Root* mRoot;
		static bool mInit;

#ifdef OGRE_STATIC_LIB
		static Ogre::StaticPluginLoader* mStaticPluginLoader;
#endif
	};

}

#endif
